@@ -36,34 +36,42 @@
 #ifndef COROSYNC_VOTEQUORUM_H_DEFINED
 #define COROSYNC_VOTEQUORUM_H_DEFINED
 
+#include <corosync/corotypes.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/**
+ * @brief votequorum_handle_t
+ */
 typedef uint64_t votequorum_handle_t;
 
-#define VOTEQUORUM_INFO_TWONODE                 1
-#define VOTEQUORUM_INFO_QUORATE                 2
-#define VOTEQUORUM_INFO_WAIT_FOR_ALL            4
-#define VOTEQUORUM_INFO_LAST_MAN_STANDING       8
-#define VOTEQUORUM_INFO_AUTO_TIE_BREAKER       16
-#define VOTEQUORUM_INFO_ALLOW_DOWNSCALE        32
-#define VOTEQUORUM_INFO_QDEVICE_REGISTERED     64
-#define VOTEQUORUM_INFO_QDEVICE_ALIVE         128
-#define VOTEQUORUM_INFO_QDEVICE_CAST_VOTE     256
-#define VOTEQUORUM_INFO_QDEVICE_MASTER_WINS   512
+#define VOTEQUORUM_INFO_TWONODE                  1
+#define VOTEQUORUM_INFO_QUORATE                  2
+#define VOTEQUORUM_INFO_WAIT_FOR_ALL             4
+#define VOTEQUORUM_INFO_LAST_MAN_STANDING        8
+#define VOTEQUORUM_INFO_AUTO_TIE_BREAKER         16
+#define VOTEQUORUM_INFO_ALLOW_DOWNSCALE          32
+#define VOTEQUORUM_INFO_QDEVICE_REGISTERED       64
+#define VOTEQUORUM_INFO_QDEVICE_ALIVE            128
+#define VOTEQUORUM_INFO_QDEVICE_CAST_VOTE        256
+#define VOTEQUORUM_INFO_QDEVICE_MASTER_WINS      512
 
-#define VOTEQUORUM_QDEVICE_NODEID               0
-#define VOTEQUORUM_QDEVICE_MAX_NAME_LEN       255
-#define VOTEQUORUM_QDEVICE_DEFAULT_TIMEOUT  10000
+#define VOTEQUORUM_QDEVICE_NODEID                0
+#define VOTEQUORUM_QDEVICE_MAX_NAME_LEN          255
+#define VOTEQUORUM_QDEVICE_DEFAULT_TIMEOUT       10000
 #define VOTEQUORUM_QDEVICE_DEFAULT_SYNC_TIMEOUT  30000
 
-#define VOTEQUORUM_NODESTATE_MEMBER             1
-#define VOTEQUORUM_NODESTATE_DEAD               2
-#define VOTEQUORUM_NODESTATE_LEAVING            3
+#define VOTEQUORUM_NODESTATE_MEMBER              1
+#define VOTEQUORUM_NODESTATE_DEAD                2
+#define VOTEQUORUM_NODESTATE_LEAVING             3
 
 /** @} */
 
+/**
+ * @brief The votequorum_info struct
+ */
 struct votequorum_info {
 	unsigned int node_id;
 	unsigned int node_state;
@@ -77,16 +85,25 @@ struct votequorum_info {
 	char qdevice_name[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
 };
 
+/**
+ * @brief The votequorum_node_t struct
+ */
 typedef struct {
 	uint32_t nodeid;
 	uint32_t state;
 } votequorum_node_t;
 
+/**
+ * @brief The votequorum_ring_id_t struct
+ */
 typedef struct {
 	uint32_t nodeid;
 	uint64_t seq;
 } votequorum_ring_id_t;
 
+/**
+ * @brief The votequorum_notification_fn_t callback
+ */
 typedef void (*votequorum_notification_fn_t) (
 	votequorum_handle_t handle,
 	uint64_t context,
@@ -95,50 +112,69 @@ typedef void (*votequorum_notification_fn_t) (
 	uint32_t node_list_entries,
 	votequorum_node_t node_list[]);
 
+/**
+ * @brief The votequorum_expectedvotes_notification_fn_t callback
+ */
 typedef void (*votequorum_expectedvotes_notification_fn_t) (
 	votequorum_handle_t handle,
 	uint64_t context,
 	uint32_t expected_votes);
 
+/**
+ * @brief The votequorum_callbacks_t struct
+ */
 typedef struct {
 	votequorum_notification_fn_t votequorum_notify_fn;
 	votequorum_expectedvotes_notification_fn_t votequorum_expectedvotes_notify_fn;
 } votequorum_callbacks_t;
 
-
 /**
- * Create a new quorum connection
+ * @brief Create a new quorum connection
+ * @param handle
+ * @param callbacks
+ * @return
  */
 cs_error_t votequorum_initialize (
 	votequorum_handle_t *handle,
 	votequorum_callbacks_t *callbacks);
 
 /**
- * Close the quorum handle
+ * @brief Close the quorum handle
+ * @param handle
+ * @return
  */
 cs_error_t votequorum_finalize (
 	votequorum_handle_t handle);
 
-
 /**
- * Dispatch messages and configuration changes
+ * @brief Dispatch messages and configuration changes
+ * @param handle
+ * @param dispatch_types
+ * @return
  */
 cs_error_t votequorum_dispatch (
 	votequorum_handle_t handle,
 	cs_dispatch_flags_t dispatch_types);
 
 /**
- * Get a file descriptor on which to poll.
+ * @brief Get a file descriptor on which to poll.
  *
- * @note votequorum_handle_t is NOT a file descriptor and may not be
- *       used directly.
+ * @note votequorum_handle_t is NOT a file descriptor and may not be used directly.
+ *
+ * @param handle
+ * @param fd
+ * @return
  */
 cs_error_t votequorum_fd_get (
 	votequorum_handle_t handle,
 	int *fd);
 
 /**
- * Get quorum information.
+ * @brief Get quorum information.
+ * @param handle
+ * @param nodeid
+ * @param info
+ * @return
  */
 cs_error_t votequorum_getinfo (
 	votequorum_handle_t handle,
@@ -146,14 +182,21 @@ cs_error_t votequorum_getinfo (
 	struct votequorum_info *info);
 
 /**
- * set expected_votes
+ * @brief set expected_votes
+ * @param handle
+ * @param expected_votes
+ * @return
  */
 cs_error_t votequorum_setexpected (
 	votequorum_handle_t handle,
 	unsigned int expected_votes);
 
 /**
- * set votes for a node
+ * @brief set votes for a node
+ * @param handle
+ * @param nodeid
+ * @param votes
+ * @return
  */
 cs_error_t votequorum_setvotes (
 	votequorum_handle_t handle,
@@ -161,45 +204,74 @@ cs_error_t votequorum_setvotes (
 	unsigned int votes);
 
 /**
- * Track node and quorum changes
+ * @brief Track node and quorum changes
+ * @param handle
+ * @param context
+ * @param flags
+ * @return
  */
 cs_error_t votequorum_trackstart (
 	votequorum_handle_t handle,
 	uint64_t context,
-	unsigned int flags );
+        unsigned int flags);
 
+/**
+ * @brief votequorum_trackstop
+ * @param handle
+ * @return
+ */
 cs_error_t votequorum_trackstop (
 	votequorum_handle_t handle);
 
 /**
- * Save and retrieve private data/context
+ * @brief Save and retrieve private data/context
+ * @param handle
+ * @param context
+ * @return
  */
 cs_error_t votequorum_context_get (
 	votequorum_handle_t handle,
 	void **context);
 
+/**
+ * @brief votequorum_context_set
+ * @param handle
+ * @param context
+ * @return
+ */
 cs_error_t votequorum_context_set (
 	votequorum_handle_t handle,
 	void *context);
 
 /**
- * Register a quorum device
+ * @brief Register a quorum device
  *
  * it will be DEAD until polled
+ *
+ * @param handle
+ * @param name
+ * @return
  */
 cs_error_t votequorum_qdevice_register (
 	votequorum_handle_t handle,
 	const char *name);
 
 /**
- * Unregister a quorum device
+ * @brief Unregister a quorum device
+ * @param handle
+ * @param name
+ * @return
  */
 cs_error_t votequorum_qdevice_unregister (
 	votequorum_handle_t handle,
 	const char *name);
 
 /**
- * Update registered name of a quorum device
+ * @brief Update registered name of a quorum device
+ * @param handle
+ * @param oldname
+ * @param newname
+ * @return
  */
 cs_error_t votequorum_qdevice_update (
 	votequorum_handle_t handle,
@@ -207,7 +279,12 @@ cs_error_t votequorum_qdevice_update (
 	const char *newname);
 
 /**
- * Poll a quorum device
+ * @brief Poll a quorum device
+ * @param handle
+ * @param name
+ * @param cast_vote
+ * @param ring_id
+ * @return
  */
 cs_error_t votequorum_qdevice_poll (
 	votequorum_handle_t handle,
@@ -216,7 +293,11 @@ cs_error_t votequorum_qdevice_poll (
 	votequorum_ring_id_t ring_id);
 
 /**
- * Allow qdevice to tell votequorum if master_wins can be enabled or not
+ * @brief Allow qdevice to tell votequorum if master_wins can be enabled or not
+ * @param handle
+ * @param name
+ * @param allow
+ * @return
  */
 cs_error_t votequorum_qdevice_master_wins (
 	votequorum_handle_t handle,
