@@ -928,13 +928,16 @@ cs_error_t cpg_zcb_alloc (
 		&res_coroipcs_zc_alloc,
 		sizeof (struct qb_ipc_response_header));
 
+	if (error != CS_OK) {
+		goto error_exit;
+	}
+
 	hdr = (struct coroipcs_zc_header *)buf;
 	hdr->map_size = map_size;
-	*buffer = ((char *)buf) + sizeof (struct coroipcs_zc_header);
+	*buffer = ((char *)buf) + sizeof (struct coroipcs_zc_header) + sizeof (struct req_lib_cpg_mcast);
 
+error_exit:
 	hdb_handle_put (&cpg_handle_t_db, handle);
-	*buffer = ((char *)*buffer) + sizeof (struct req_lib_cpg_mcast);
-
 	return (error);
 }
 
@@ -943,11 +946,12 @@ cs_error_t cpg_zcb_free (
 	void *buffer)
 {
 	cs_error_t error;
+	unsigned int res;
 	struct cpg_inst *cpg_inst;
 	mar_req_coroipcc_zc_free_t req_coroipcc_zc_free;
 	struct qb_ipc_response_header res_coroipcs_zc_free;
 	struct iovec iovec;
-	struct coroipcs_zc_header *header = (struct coroipcs_zc_header *)((char *)buffer - sizeof (struct coroipcs_zc_header));
+	struct coroipcs_zc_header *header = (struct coroipcs_zc_header *)((char *)buffer - sizeof (struct coroipcs_zc_header) - sizeof (struct req_lib_cpg_mcast));
 
 	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
@@ -969,8 +973,18 @@ cs_error_t cpg_zcb_free (
 		&res_coroipcs_zc_free,
 		sizeof (struct qb_ipc_response_header));
 
-	munmap ((void *)header, header->map_size);
+	if (error != CS_OK) {
+		goto error_exit;
+	}
 
+	res = munmap ((void *)header, header->map_size);
+	if (res == -1) {
+		error = qb_to_cs_error(-errno);
+
+		goto error_exit;
+	}
+
+error_exit:
 	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
